
%
% KeCoDe_ENCODER_ACHROM applies DCT-based image coding algorithms to the
% input 256*256 gray scale image to obtain a file with the desired
% entropy (in bits/pixel).
%
% The program stores the encoded file in the folder\name provided by the user.
% Particular appendices to the given file name will be added depending on the
% selected algorithm (see names below).
%
% The encoded file can be decoded with KeCoDe_DECODER.M
% Besides, the encoder routine also gives the decoded image, the entropy
% of the code (in bits/pix) and a number of distortion measures.
%
% Algorithms 1-4 below use quantizers based on human vision models of increasing
% accuracy. Algorithms 5-7 are based on similar vision models and also use
% SVM based selection of transform coefficients.
%
%  * MODEL 1: JPEG91.   Wallace, Comm. of the ACM, Vol.34(4):30-44, (1991)
%             block DCT transform coding with quantization based on JPEG-like
%             linear CSF vision model. It does not include masking relations at all.
%             The name of the encoded file will be: name_jpeg91_entropy_1.zip
%
%             Note, though, that this is not exactly the JPEG standard since
%             (1) 16*16 DCT blocks are used, (2) run length encoding, DPCM DC encoding
%             and block information arrangement are done in a non-standard way, and (3)
%             the final entropy coding is done by the Matlab zip routine.
%             Nevertheless this implementation is the appropriate for fair
%             comparison with the rest of the algorithms since the above (non-fundamental)
%             details are implemented in the same way.
%
%  * MODEL 2: Malo99.   Malo et al., Electr. Lett., Vol.35(13):1067-1068, (1999)
%             block DCT transform coding with quantization based on a
%             simple (point-wise) non-linear masking model. It includes
%             auto-masking but it does not include masking relations among
%             coefficients.
%             The name of the encoded file will be: name_malo99_entropy_1.zip
%
%  * MODEL 3: Malo06.   Malo et al. IEEE Trans. Im. Proc., Vol.15(1):68-80 (2006)
%             block DCT transform plus non-linear divisive normalization
%             transform and uniform quantization. This is the proper way to take
%             frequency selectivity and all the masking relations into account in
%             the quantization process.
%             The name of the encoded file will be:  name_malo06_entropy_1.zip
%
%  * MODEL 4: Robinson03. Robinson & Kecman, IEEE Trans. Neur.Nets., Vol.14(4):950-958 (2003)
%             block DCT transform plus CSF inspired constant insensitivity
%             SVM coefficient selection (RKi-1). SVM based on a rough
%             linear vision model.
%             The name of the encoded file will be:  name_robinson03_entropy_1.zip
%
%  * MODEL 5: Gomez05.   Gomez et al., IEEE Trans. Neur. Nets., Vol.16(6):1574-1581 (2005)
%             block DCT transform coding plus CSF adaptive insensitivity
%             SVM coefficient selection. SVM based on an accurate linear vision
%             model.
%             The name of the encoded file will be:  name_gomez05_entropy_1.zip
%
%  * MODEL 6: Camps08.   Camps et al., J. Mach. Learn. Res., Vol.9(1):49-66 (2008)
%             block DCT transform coding plus divisive normalization and
%             constant insensitivity SVM coefficient selection.  SVM trained in a
%             vision model domain that takes into account frequency seletivity and
%             masking relations among coefficients.
%             The name of the encoded file will be:  name_camps08_entropy_1.zip
%
% The bit rate of the above algorithms is controlled by different
% parameters:
%
%   * Algorithms 1-3 depend on the Control Parameter, 'CP'.
%     Smaller CP values imply more coarse quantization thus giving smaller
%     files and more distorted images.
%
%   * Algorithms 4-6 depend on two parameters:
%     (1) the insensitivity parameter of the SVM, 'Epsilon'.
%     (2) the number of bits used to encode the SVM weights, 'Bits'.
%     For a fixed number of bits, smaller Epsilon values imply keeping more
%     support vectors (or coefficients) and hence larger files and better
%     quality images.
%
% The user has to provide a target entropy value. The program then
% sets the values of the control parameters (CP, or Epsilon and Bits) and
% iteratively modifies them to achieve the target entropy for the particular image.
%
% SYNTAX:
% [Results] = KeCoDe_encoder_achrom(Im,MODEL,'output_folder','name',target_entropies,Num_iterat)
%
% Input variables:
% ----------------
%  * Im             :  256*256 image matrix double precision numbers in the range [0 255]
%  * MODEL          :  1-6
%  * 'output_folder':  String with the folder where it will be written the output file(s)
%  * 'name'         :  String with the name of the output file
%                      Note that an appendix to this name will be added
%                      depending on the coding algorithm.
%  * target_entropies: Vector containing the set of target entropies
%                      (an image can be compressed at different entropies
%                      with a single call to this function)
%                      There will be as many output files as target entropy
%                      values.
%  * Num_iterat     :  Number of iterations to look for the target entropy
%
% Output:
% -------
%  * Results        :  Struct variable with the following fields
%
%       - Results(i).Image   = Decoded image corresponding to the i-th value
%                              of the target entropy vector.
%       - Results(i).Entropy = Entropy (in bits/pix): file_size/256^2
%       - Results(i).RMSE    = RMSE distortion of the i-th decoded image
%       - Results(i).SSIM    = Structural SIMilarity Index of the i-th
%                              decoded image.
%                              (See Wang et al. IEEE Tr. Im. Proc., 2004
%                              for a description of this distortion measure)
%       - Results(i).MPE_linear= Maximum Perceptual Error of the i-th decoded
%                              image based on a linear CSF vision model.
%                              (See Gomez et al. IEEE Tr. Neur. Nets., 2005
%                              for a description of this distortion measure)
%       - Results(i).MPE_non_linear= Maximum Perceptual Error of the i-th decoded
%                              image based on a non linear vision model.
%                              (See Camps et al. JMLR, 2008 for a description
%                              of this distortion measure)
%
function [Results] = KeCoDe_encoder_achrom(Im,algorit,directorio,ficherin,entropias,N_it)
warning('off','MATLAB:dispatcher:InexactMatch')
Im=double(Im);
if algorit==1
    algoritmo=6;

    Desired_entropy=[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 1   1.2 1.4 1.6  1.8 2  2.2];
    Approximate_uCP=[1.5 2.6 3.3 3.8 4.4 4.9 5.3 5.7 6.5 7.3 8.1 9    9.9 11 12.2];
    for i=1:length(entropias)
        if entropias(i)>max(Desired_entropy),
            parametro(i)=12.5;
        elseif entropias(i)<min(Desired_entropy),
            parametro(i)=1.2;
        else
            parametro(i)=interp1(Desired_entropy,Approximate_uCP,entropias(i));
        end
    end
elseif algorit==2
    algoritmo=7;

    Desired_entropy=[ 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 1   1.2 1.4 1.6  1.8 2   2.2];
    Approximate_uCP=[ 1.3 2.3 3   3.3 3.7 4.1 4.5 4.8 5.3 5.8 6.3 6.8  7.4 7.8 8.3];
    for i=1:length(entropias)
        if entropias(i)>max(Desired_entropy),
            parametro(i)=8.4;
        elseif entropias(i)<min(Desired_entropy),
            parametro(i)=1.1;
        else
            parametro(i)=interp1(Desired_entropy,Approximate_uCP,entropias(i));
        end
    end
elseif algorit==3
    algoritmo=9;

    Desired_entropy=[    0.2 0.3 0.4 0.5 0.6 0.7 0.8 1   1.2 1.4 1.6 1.8 2   2.2];
    Approximate_uCP=[    0.3 1.0 1.7 2.1 2.6 2.9 3.3 3.9 4.4 4.9 5.3 5.6 6   6.3];
    for i=1:length(entropias)
        if entropias(i)>max(Desired_entropy),
            parametro(i)=6.5;
        elseif entropias(i)<min(Desired_entropy),
            parametro(i)=0.15;
        else
            parametro(i)=interp1(Desired_entropy,Approximate_uCP,entropias(i));
        end
    end
elseif algorit==4
    algoritmo=2;

    Desired_entropy=[ 0.1  0.2 0.3  0.4  0.5   0.6   0.7   0.8   1     1.2    1.4    1.6    1.8    2      2.2];
    Approx_Epsilon=[ 23.6 2.9 0.46 0.69 0.012 0.037 0.002 0.016 0.013 0.0004 0.0034 0.0053 0.0016 0.0037 0.0017];
    for i=1:length(entropias)
        if entropias(i)>max(Desired_entropy),
            parametro(i)=0.001;
        elseif entropias(i)<min(Desired_entropy),
            parametro(i)=25;
        else
            parametro(i)=interp1(Desired_entropy,Approx_Epsilon,entropias(i));
        end
    end
elseif algorit==5
    algoritmo=3;

    Desired_entropy=[0.1  0.2  0.3  0.4   0.5   0.6   0.7   0.8   1     1.2    1.4   1.6   1.8    2      2.2];
    Approx_Epsilon=[15.7 0.18 0.10 0.065 0.009 0.026 0.002 0.012 0.011 0.0003 0.003 0.005 0.0015 0.0033 0.0016];
    for i=1:length(entropias)
        if entropias(i)>max(Desired_entropy),
            parametro(i)=0.001;
        elseif entropias(i)<min(Desired_entropy),
            parametro(i)=18;
        else
            parametro(i)=interp1(Desired_entropy,Approx_Epsilon,entropias(i));
        end
    end
elseif algorit==6
    algoritmo=4;

    Desired_entropy=[0.1  0.2  0.3  0.4   0.5   0.6   0.7   0.8   1     1.2    1.4    1.6   1.8    2      2.2];
    Approx_Epsilon=[0.79 0.36 0.17 0.08  0.025 0.025 0.004 0.01  0.004 0.0027 0.0024 0.003 0.0036 0.0002 0.0012];
    for i=1:length(entropias)
        if entropias(i)>max(Desired_entropy),
            parametro(i)=0.001;
        elseif entropias(i)<min(Desired_entropy),
            parametro(i)=1;
        else
            parametro(i)=interp1(Desired_entropy,Approx_Epsilon,entropias(i));
        end
    end
else
    disp('Not a valid algorithm selection')
    algoritmo=1000;
end
if algoritmo<100

    [perfil,K,exponente] = computing_parameters_entropy(algoritmo);
    if(algoritmo <= 4)
        Results = entropy_svr(algoritmo,entropias,Im,parametro,perfil,K,exponente,directorio,ficherin,N_it);
    else
        Results = entropy_ucp(algoritmo,entropias,Im,parametro,exponente,directorio,ficherin,N_it);
    end
end
