
%
% KeCoDe_ENCODER_COLOR applies DCT-based image coding algorithms to the
% input 256*256*3 RGB color image to obtain a file with the desired
% entropy (in bits/pixel).
%
% The program stores the encoded file in the folder\name provided by the user.
% Particular appendices to the given file name will be added depending on the
% selected algorithm (see names below).
%
% The encoded file can be decoded with KeCoDe_DECODER_COLOR.M
% Besides, the encoder routine also gives the decoded image, the entropy
% of the code (in bits/pix) and a number of distortion measures.
%
%  * MODEL 1: JPEG91.   Wallace, Comm. of the ACM, Vol.34(4):30-44, (1991)
%             block DCT transform coding with quantization based on JPEG-like
%             linear CSF vision model in each chromatic channel (YUV).
%             It does not include masking relations at all.
%             The name of the encoded file will be: name_jpeg91_c_entropy_1.zip
%
%             Note, though, that this is not exactly the JPEG standard since
%             (1) 16*16 DCT blocks are used, (2) run length encoding, DPCM DC encoding
%             and block information arrangement are done in a non-standard way, and (3)
%             the final entropy coding is done by the Matlab zip routine.
%             Nevertheless this implementation is the appropriate for fair
%             comparison with the other algorithm since the above (non-fundamental)
%             details are implemented in the same way.
%
%  * MODEL 2: Gutierrez11. Gutierrez et al., Recent Patents in Signal Proc., In press (2011)
%             block DCT transform coding plus chromatic divisive normalization in each
%             YUV channel and constant insensitivity SVM coefficient selection.
%             Dedicated SVMs are trained in a vision model domain that takes into
%             account both frequency seletivity and masking relations among coefficients.
%             The name of the encoded file will be:  name_gutierrez09_c_entropy_1.zip
%
% The bit rate of the above algorithms is controlled by different
% parameters:
%
%   * Algorithm 1 depend on the Control Parameter, 'CP'.
%     Smaller CP values imply more coarse quantization thus giving smaller
%     files and more distorted images.
%
%   * Algorithm 2 depend on two parameters:
%     (1) the insensitivity parameter of the SVM, 'Epsilon'.
%     (2) the number of bits used to encode the SVM weights, 'Bits'.
%     For a fixed number of bits, smaller Epsilon values imply keeping more
%     support vectors (or useful coefficients) and hence larger files and better
%     quality images.
%
% The user has to provide a target entropy value. The program then
% sets the values of the control parameters (CP, or Epsilon and Bits) and
% iteratively modifies them to achieve the target entropy for the particular image.
%
% SYNTAX:
% [Results] = KeCoDe_encoder_color(Im,MODEL,'output_folder','name',target_entropies,Num_iterat)
%
% Input variables:
% ----------------
%  * Im             :  256*256*3 image matrix double precision numbers in the range [0 255]
%  * MODEL          :  1-2
%  * 'output_folder':  String with the folder where it will be written the output file(s)
%  * 'name'         :  String with the name of the output file
%                      Note that an appendix to this name will be added
%                      depending on the coding algorithm.
%  * target_entropies: Vector containing the set of target entropies
%                      (an image can be compressed at different entropies
%                      with a single call to this function)
%                      There will be as many output files as target entropy
%                      values.
%  * Num_iterat     :  Number of iterations to look for the target entropy
%
% Output:
% -------
%  * Results        :  Struct variable with the following fields
%
%       - Results(i).Image   = Decoded image corresponding to the i-th value
%                              of the target entropy vector.
%       - Results(i).Entropy = Entropy (in bits/pix): file_size/256^2
%       - Results(i).RMSE    = RMSE distortion of the i-th decoded image
%       - Results(i).SSIM    = Structural SIMilarity Index of the i-th
%                              decoded image.
%                              (See Wang et al. IEEE Tr. Im. Proc., 2004
%                              for a description of this distortion measure)
%       - Results(i).SCIELab_exp1= Spatial CIE Lab distortion with summation exponent
%                              equal to 1, default in S-CIELab computations.
%                              (See Zhang & Wandell, Sig. Proc. 1998
%                              for a description of this distortion measure)
%       - Results(i).SCIELab_exp2= Spatial CIE Lab distortion with summation exponent
%                              equal to 2. (See Zhang & Wandell, Sig. Proc. 1998
%                              for a description of this distortion
%                              measure)
%
function [Results] = KeCoDe_encoder_color(Im,algorit,directorio,ficherin,entropias,N_it)
warning('off','MATLAB:dispatcher:InexactMatch')
Im=double(Im);
if algorit==1

    Desired_entropy=[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 1   1.2 1.4 1.6  1.8 2   2.2];
    Approximate_uCP=[1.3 3.1 4   4.7 5.3 5.7 6.2 6.5 7.2 7.7 8.2 8.7  9.1 9.5 9.8];
    for i=1:length(entropias)
        if entropias(i)>max(Desired_entropy),
           parametro(i)=10.5;
        elseif entropias(i)<min(Desired_entropy),
           parametro(i)=1.1;
        else
           parametro(i)=interp1(Desired_entropy,Approximate_uCP,entropias(i));
        end
    end
elseif algorit==2
 Desired_entropy=[0.1  0.2  0.3  0.4   0.5   0.6   0.7   0.8    1      1.2    1.4    1.6     1.8     2       2.2];
 Approx_Epsilon= [1.6  0.23 0.06 0.02  0.006 0.004 0.002 0.0016 0.0009 0.0005 0.0003 0.00016 0.00017 0.00012 0.0001];
 Approx_bits   = [2.5  2.5  2.5  2.5   2.5   2.5   3     3.5    4.5    5      6      6.5     7       7.5     8];
    for i=1:length(entropias)
        if entropias(i)>max(Desired_entropy),
           parametro(i)=0.00005;
           bits_in(i)=8;
        elseif entropias(i)<min(Desired_entropy),
           parametro(i)=1.9;
           bits_in(i)=2;
        else
           parametro(i)=interp1(Desired_entropy,Approx_Epsilon,entropias(i));
           bits_in(i)=interp1(Desired_entropy,Approx_bits,entropias(i));
        end
    end
else
   disp('Not a valid algorithm selection')
   algorit=1000;
end
if algorit<=2
    normaliz=1;
    To=[100 0 0];
    dUmax=40;
    dVmax=43;
    niveles=1;
    if algorit==1
        Results=entropy_jpeg(Im,normaliz,To,dUmax,dVmax,[parametro' parametro' parametro'],entropias,directorio,ficherin,N_it,niveles);
    else
        K=kernel2d_variance(16,1);
        exponente=[2 2 2];
        tipo_alfa=1;
        Results=entropy_nl_svr(Im,normaliz,To,dUmax,dVmax,[parametro' parametro' parametro'],bits_in,K,exponente,tipo_alfa,entropias,directorio,ficherin,N_it);
    end
end

