
% MALO06: image transform coding by using a non-linearly transformed DCT
% using divisive normalization (see Malo et al. 06)
%
% MALO06 encodes 256*256 images by first transforming each 16*16 image block
% into a non-linear domain (DCT+divisive normalization) that simultaneously
% reduces the statistical and the perceptual relations among coefficients.
% Then, each transformed block is quantized using uniform bit allocation and uniform
% quantizers because the non-linear domain is Euclidean from the perceptual point
% of view.
%
% Two versions of the algorithm are available. These versions differ in the
% parameters of the non-linear perceptual transform (the divisive normalization).
% The different versions can be selected using the parameter 'exp':
%
%    exp = 1    Parameter set with robust inverse
%    exp = 2    Parameter set with better results at high compression (less than 0.3 bpp)
%               but more unstable inverse
%
% (see qdct65.m for details)
%
% In this implementation the allocated bits are controlled by the
% control parameter, CP.
% Increasing CP reduces the quantization step sizes, increasing the entropy,
% and reducing the distortion.
%
% The algorithm starts by using a CP value provided by the user, uCP.
% This CP value is iteratively modified till the desired entropy is achieved.
%
% USE: [Results] = malo06_2(Im,exp,parametro,B,variable)
%
% INPUT:
%
% Im        -> Image
% parametro -> Distortion parameter value
% B         -> Control parameter
% variable  -> Text string to choose the distortion parameter
%
% OUTPUT:
%
% Results   -> Struct with the information a bout results
%              to the diostortion parameter:
%              struct('uCP',B,'Image',ImR,'Entropy',H)
%              Results.SNR, Results.MSE, Results.SSIM, Results.MPE_lineal,
%              Results.MPE_no_lineal, Results.MPE_no_linea_JMLR
%
% REFERENCE:
%
%    Malo et al., IEEE Tr. Im. Proc., 2006.
%

function [Results] = malo06_ucp_distortion(Im,exp,param,B_inicial,variable);

Im=double(Im);

Maximo=0.8;
cero=1;
blo=[];
for i=1:16
    for j=1:16
        blo=[blo;i j];
    end
end
norma=[3 mean(mean(Im))];
calc_eig=0;
calc_eig_sin_q=0;

Results=[];

contador = 0;

for parametro = param

    disp(['Adjustment of uCP to reach desired ' variable ':   ' num2str(parametro)]);

    switch(variable)
        case 'SNR', parametro_actual = 1; umbral = 0.05;
        case 'MSE', parametro_actual = 1; umbral = 0.5;
        case 'SSIM', parametro_actual = 0; umbral = 0.001;
        case 'MPE lineal', parametro_actual = 100; umbral = 0.01;
        case 'MPE no lineal', parametro_actual = 100; umbral = 0.01;
        case 'MPE no lineal JMLR', parametro_actual = 100; umbral = 0.1;
    end

    iter = 0;

    B = 0;

    porcentaje = 1;

    if(isequal(variable,'SSIM'))
        sentido = 1;
    else
        sentido = 0;
    end

    no_results = 0;

    l_min = 0;
    l_max = 100000;

    while(abs(parametro - parametro_actual) > umbral) & (iter < 30)

        iter = iter + 1;

        if(iter ~= 1 & iter ~= 2)
            switch(variable)
                case 'SNR',
                    [B, l_min, l_max, porcentaje, sentido] = fitting_uCP_SSIM(parametro, parametro_actual, B, l_min, l_max, porcentaje, sentido);
                case 'MSE',
                    [B, l_min, l_max, porcentaje, sentido] = fitting_uCP(parametro, parametro_actual, B, l_min, l_max, porcentaje, sentido);
                case 'SSIM',
                    [B, l_min, l_max, porcentaje, sentido] = fitting_uCP_SSIM(parametro, parametro_actual, B, l_min, l_max, porcentaje, sentido);
                case 'MPE lineal',
                    [B, l_min, l_max, porcentaje, sentido] = fitting_uCP(parametro, parametro_actual, B, l_min, l_max, porcentaje, sentido);
                case 'MPE no lineal',
                    [B, l_min, l_max, porcentaje, sentido] = fitting_uCP(parametro, parametro_actual, B, l_min, l_max, porcentaje, sentido);
                case 'MPE no lineal JMLR',
                    [B, l_min, l_max, porcentaje, sentido] = fitting_uCP(parametro, parametro_actual, B, l_min, l_max, porcentaje, sentido);
            end
        end

        [imq1,imq2,codig,signos,codigo,nceros,DC,Hp,Hi,meig,mceig,values,valuesc]=qdct65(Im,B,Maximo,norma,blo,exp,cero,calc_eig,calc_eig_sin_q);
        ImR = real(imq1);

        switch(variable)
            case 'SNR',
                [mse,SNRi] = emse2(Im,ImR);
                fprintf('   It. =
                parametro_actual = SNRi;
            case 'MSE',
                [mse,SNRi] = emse2(Im,ImR);
                fprintf('   It. =
                parametro_actual = mse;
            case 'SSIM',
                esesim = ssim_index(Im,ImR);
                fprintf('   It. =
                parametro_actual = esesim;
            case 'MPE lineal',
                [MSE,d_l,dx0,MPEx0,MPEoo0,MPEexpesp0] = disdis_lineal(Im,ImR,2,[3 mean(mean(Im))],2,2);
                mpelin = d_l;
                fprintf('   It. =
                parametro_actual = mpelin;
            case 'MPE no lineal',
                [MSE,d,dx,MPEx,MPEoo,MPEexpesp] = disdis7(Im,ImR,[3 mean(mean(Im))],2,1,2,2);
                mpenonlin= d;
                fprintf('   It. =
                parametro_actual = mpenonlin;
            case 'MPE no lineal JMLR',
                [MSE,d,dx,MPEx,MPEoo,MPEexpesp] = disdis7(Im,ImR,[3 mean(mean(Im))],1,0.5,2,2);
                mpenonlinJMLR = d;
                fprintf('   It. =
                parametro_actual = mpenonlinJMLR;
        end

        if(~isequal(variable,'SSIM') & (iter == 1) & (parametro - parametro_actual > 0) & (B == 0))
            no_results = 1;
            disp('No se puede ajustar el uCP para esta distorsion');
            break;

        elseif(isequal(variable,'SSIM') & (iter == 1) & (parametro - parametro_actual < 0) & (B == 0))
            no_results = 1;
            disp('No se puede ajustar el uCP para esta distorsion');
            break;

        elseif((iter == 1) & (abs(parametro - parametro_actual) <= umbral) & (B == 0))
            no_results = 1;
            disp('No se puede ajustar el uCP para esta distorsion');
            break;

        elseif(iter == 1)
            B = B_inicial;
        end
    end

    if(no_results == 0)

        contador = contador + 1;

        H = Hi/(size(Im,1) * size(Im,2));

        Results(contador).uCP = B;
        Results(contador).Image = ImR;
        Results(contador).Entropy = H;

        switch(variable)
            case 'SNR',
                Results(contador).SNR = SNRi;
            case 'MSE',
                Results(contador).MSE = mse;
            case 'SSIM',
                Results(contador).SSIM = esesim;
            case 'MPE lineal',
                Results(contador).MPE_lineal = mpelin;
            case 'MPE no lineal',
                Results(contador).MPE_no_lineal = mpenonlin;
            case 'MPE no lineal JMLR',
                Results(contador).MPE_no_lineal_JMLR = mpenonlinJMLR;
        end
    end
end
