%
% KeCoDe_denosing applies wavelet-based image denoising algorithms to the
% input 256*256 gray scale image. This version is thought only for gaussian
% noise. Note that the kernel denoising colud be applied to different
% kinds of noise, however some parameters have to be precomputed (see
% model 4 or [Laparra10] for details).
%
% The available denoising algorithms and the corresponding Matlab functions are:
%
%   * model 1 - Hard Thresholding [Donoho95,Simoncelli99].... hardthres.m
%       Assumes the noise variance is known.
%       Applies the threshold=3*sigma_n rule.
%
%   * model 2 - Soft Thresholding [Donoho95,Simoncelli99].... softthres.m
%       Assumes the noise variance is known.
%       Applies the threshold=3*sigma_n rule.
%
%   * model 3 - Bayesian denoising assuming (1) Gaussian noise of known variance
%     and, (2) Gaussian marginals for the wavelet coefficients of natural
%     images [Figueiredo01]........................ bayesian_gauss_margin.m
%
%   * model 4 - Kernel regularization in the wavelet domain with kernels inspired in
%     mutual information measures [Laparra10] ..... mi_kernel_denoising.m
%
%     NOTE: The function 'mi_kernel_denoising.m' admits precomputed parameters
%     in order to apply the algorithm in different kinds of noise. See the
%     JPEG example below.
%
%     In 'KeCoDe_denoising.m' the algorithm is prepared to work with
%     Gaussian nosie. In order to use 'mi_kernel_denoising.m' on different
%     kind noise it requires some knowledge on the signal and
%     the noise:
%     1- Average variance of the signal in the wavelet domain to set
%        the SVM penalization profile, C_i. This variance profile is an
%        intrinsic feature of natural images and can be estimated from
%        a natural image database.
%     2- Average standard deviation of the noise in the wavelet domain to
%        set the insensitivity profile, epsilon_i. This deviation profile
%        can be estimated from noise samples.
%     3- 2D histogram of the noise in the spatial domain to look for the
%        SVM parameters that better identify the noise. SVM parameter
%        optimization can be done by minimizing the KL divergence between
%        the known noise histogram and the histogram of the estimated
%        noise.
%     The code provided in KeCoDe skips all the off-line
%     preprocessing steps:
%     1- Average variance of natural images in the steerable domain was
%        precomputed from the McGill natural image data base
%        (http://tabby.vision.mcgill.ca/) and stored in the file:
%                "C_profile.mat"
%     2- Average standard deviation profile of Gaussian noise and
%        JPEG noise was precomputed from (1) noise realizations and
%        (2) JPEG compressed (at MATLAB quality compression Q=7) examples
%        respectively, and stored in the files:
%    		"Epsilon_profile_Gaussian.mat"
%    		"Epsilon_profile_JPEG_Q7.mat"
%        If the Q factor of the JPEG quantization is modified, or another noise
%        source is present, the epsilon profile should be computed by looking at
%        the average wavelet transform of noise samples.
%        For signal independent noise sources, the unit variance profile
%        scales as: epsilon_profile=sqrt(Var)*unit_variance_epsilon_profile..
%        Therefore, for white noise Gaussian sources the code provided here
%        is general assuming known variance (as the other algorithms).
%	  3- KLD-based SVR selection is not implemented in the code. Instead, fixed
%        (near optimum) values are given in each case (Gaussian and JPEG), in the
%        ranges described in [Laparra08].
%
% [Laparra10]    V. Laparra, J. Gutierrez, G. Camps and J. Malo
%                Image Denoising with Kernels based on Natural Image Relations
%                Journal of Machine Learning Research 11(Feb):873?903, 2010
%
% [Donoho95]     David L. Donoho and Iain M. Johnstone. Adapting to unknown
%                smoothness via wavelet shrinkage. J. Am. Stat. Assoc., 90:1200-1224, 1995.
%
% [Simoncelli99] E. Simoncelli. Bayesian denoising of visual images in the wavelet
%                domain. In Bayesian Inference in Wavelet Based Models, pages 291-308.
%                Springer-Verlag, NY, 1999.
%
% [Figueiredo01] M. Figueiredo and R. Nowak. Wavelet-based image estimation:
%                an empirical Bayes approach using Jeffrey's noninformative prior.
%                IEEE Transactions on Image Processing, 10(9):1322-1331, 2001.
%
% [Simoncelli97] E. Simoncelli. MatlabPyrTools. Matlab toolbox for wavelet transforms
%                http://www.cns.nyu.edu/~lcv/software.php
%
% [Foi06]        Alessandro Foi. Routine  'function_stdEst2D.m'  for automatic
%                noise variance estimation. Tampere University of
%                Technology - 2005-2006
%                http://www.cs.tut.fi/~lasip/2D
%
% SYNTAX:
% Im_d = KeCoDe_denoising(Im,model,var)
%
% Input variables:
% ----------------
%  * Im_n           :  256*256 noisy image matrix double precision numbers in the range [0 255]
%  * MODEL          :  1-4 (see above models explanation)
%  * Var            :  Variance of Gaussian noise. This parameter is optinal.
%                      If it is not provided, the variance will be estimated using
%                      'function_stdEst2D.m' [Foi06]
%
% Output:
% -------
%  * Im_d           :  Denoised image
%
function Im_d = KeCoDe_denoising(Im_n,model,var)
warning('off','MATLAB:dispatcher:InexactMatch')
Im_n=double(Im_n);
if exist('var')
else
    dev = function_stdEst2D(Im_n);
    var = dev^2;
end
if model==1
    Im_d = hardthres(Im_n,var);
elseif model==2
    Im_d = softthres(Im_n,var);
elseif model==3
    Im_d = bayesian_gauss_margin(Im_n,var);
elseif model==4
    C       = 3000;
    S       = 4;
    epsilon = 2;
    load Epsilon_profile_Gaussian.mat
    epsilon_profile  = perfil_eps*sqrt(var);
    Im_d = mi_kernel_denoising(Im_n,epsilon,C,S,epsilon_profile);
end
