%
%   DEGRADE an image using a PSF spatially invariant h (Gaussian low-pass filter
%   with normalized cutoff frequency fc, 0<fc<1 -1 is the Nyquist frequency-).
%   Also, it includes isotropic colored noise in the 0<[f1,f2]<1 band of a
%   given variance.
%
%   NOTE THAT: In order to allow frequencies higher than fc, there is a parameter
%   'val' in the frequency response.
%   Since we are normlaizing the PSF in order to keep the DC component, the value
%   of the frequency response for high frequencies is not 'val' but slightly higher.
%   In any case, the function gives back H (frequency response of the blurring filter)
%   With val=0 no the normalization is not modified.
%
%   The degraded image is:
%    Id = conv2(Im,h) + noise;
%
%
%    INPUTS:
%
%    Im: Dergraded image.
%    fcorte: Cutoff PSF frequency
%    val: value to control the response value above fc.
%    [f1 f2]: Vector of frecuencies defining the band of the colored noise.
%    vari: variance of the aditive noise.
%
%    OUTPUTS:
%
%    Id:  Degraded image
%    PSF: Peak Spread Function
%    H: Frequency response H=abs(freqz2(PSF));
%    noise: added noise
%    blurred: Blurred image (before adding the noise)
%
%    USE:
%    [Id,PSF,H,noise,Blurred] = degrade2(Im,fcorte,val,[f1 f2],vari);

function [Id,PSF,H,noise,Blurred]=degrade2(Im,fc,Val_min,lim_frec,vari);

N=size(Im);

Numpf = 11;

[f1,f2] = freqspace(Numpf,'meshgrid');
Hd = ones(Numpf);
r = sqrt(f1.^2 + f2.^2);
Hd(r>fc) = Val_min;
win = fspecial('gaussian',Numpf,2);
win = win ./ max(win(:));
PSF = fwind2(Hd,win);
PSF=PSF/sum(sum(PSF));
H=abs(freqz2(PSF));

Im2 = ampliaconborde(Im,10);

Im2 = conv2(Im2,PSF,'same');

Blurred = Im2(11:11+N(1)-1,11:11+N(2)-1);

[f1,f2] = freqspace(N,'meshgrid');
r = sqrt(f1.^2 + f2.^2);
Hruido=(r>=lim_frec(1))&(r<=lim_frec(2));
tfruido=Hruido.*exp(sqrt(-1)*2*pi*rand(N));
ruido=real(ifft2(fftshift(tfruido)));
varruido=var(ruido(:));

noise = sqrt(vari)*ruido/sqrt(varruido);

Id = Blurred + noise;
