% HARDTHRES image denoising by hard thresholding in the wavelet domain.
%
% This routine uses 4 scales QMF orthogonal wavelets (see MatlabPyrTools).
% By default it uses the 3*sigma_n rule to set the threshold.
% The user may specify a different threshold factor, i.e.
% thres=thres_factor*sigma_n.
%
% Hard thresholding [Donoho95] can be derived in a Bayesian framework using
% a very specific combination of noise and image models (MAP estimation of
% Generalized Laplacian PDF signal with particular kurtosis (k=0.5) assuming
% Gaussian noise [Simoncelli99]).
%
% SYNTAX:
%
%       im_r = hardthres(im_n,variance,thres_factor);
%
%  Input
%  -----
%    * im_n         = noisy image
%    * variance     = noise variance
%    * thres_factor = factor on the noise deviation to set the threshold (optional!)
%                     If no factor is provided the 3*sigma_n rule is applied.
%  Output
%  ------
%    * im_r         = denoised image
%
% REFERENCES:
%
% [Donoho95]     David L. Donoho and Iain M. Johnstone. Adapting to unknown
%                smoothness via wavelet shrinkage. J. Am. Stat. Assoc., 90:1200-1224, 1995.
% [Simoncelli99] E. Simoncelli. Bayesian denoising of visual images in the wavelet
%                domain. In Bayesian Inference in Wavelet Based Models, pages 291-308.
%                Springer-Verlag, NY, 1999.
% [Simoncelli97] E. Simoncelli. MatlabPyrTools. Matlab toolbox for wavelet transforms
%                http://www.cns.nyu.edu/~lcv/software.php

function A=hardthres(Im,var,varargin)

NW=4;

[pyr,ind]=buildWpyr(Im,NW);
pyr2=abs(pyr);
pyr_sg=sign(pyr);

if nargin==2
   res=3*sqrt(var);
else
   res=varargin{1}*sqrt(var);
end

PYR2=zeros(1,length(pyr2));
PYR2(find(pyr2>res))=pyr2(find(pyr2>res));

PYR2(65281:65536)=pyr(65281:65536);
PYR3=PYR2.*pyr_sg';

A=reconWpyr(PYR3',ind);
