function  KeCoDe_coder_tools_demo_achrom

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                       %
%%  KeCoDe  Demo 1:                      %
%%  Encoding-Decoding Achromatic Images  %
%%                                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     Read and execute this demo to learn how to encode-decode 256*256 achromatic images
%     using KeCoDe toolbox
%
%     The demo compares 6 algorithms:
%         - JPEG (with CSF quantization matrix) linear human visual system model
%         - Malo99: point-wise non-linear human visual system model
%         - Malo06: general non-linear human visual system model
%         - Robinson03: SVR + simplified CSF
%         - Gomez05: SVR + accurate CSF
%         - Camps08: general non-linear human visual system model + SVR
%
%     It generates six files of the same size at the tmp subfolder in the KeCoDe folder
%     and shows the results of the decoded images along with three distortion measures
%     RMSE, SSIM and Divisive Normalization based metric (d_DCT).
%
%     Target entropy is 0.55 bits/pix
%
%     This demo is illustrative but slow: it takes about 11 min to complete
%     It is slow due to the loop to obtain the required entropy with every
%     algorithm.
%
%     USE:  KeCoDe_coder_tools_demo_achrom
%
%     Related papers:
%
%   [Malo99]  J.Malo, F.Ferri, J.Albert, J.Soret
%             Comparison of perceptually uniform quantization with average
%             perceptual error minimization in image transform coding.
%             Electronics Letters , Vol. 35, 13, pp. 1067-1068 (1999)
%
%   [Malo00a] J.Malo, F.Ferri, J.Albert, J.Soret, J.M.Artigas
%             The role of perceptual contrast non-linearities in image
%             transform quantization.
%             Image & Vision Computing, Vol. 18, 3, pp. 233-246 (2000)
%
%   [Malo01]  J.Malo, J.Gutierrez, I.Epifanio, F.Ferri, J.M.Artigas
%             Perceptual feed-back in multigrid motion estimation using an
%             improved DCT quantization.
%             IEEE Trans. Im. Proc. Vol. 10, 10, pp. 1411-1427 (2001)
%
%   [Gomez05] G. Gomez, G. Camps, J. Gutiérrez and J. Malo
%             Perceptual Adaptive Insensitivity for Support Vector
%             Machine Image Coding.
%             IEEE Trans. Neural Networks Vol. 16, 6, pp 1574-1581 (2005)
%
%   [Malo06]  J.Malo, I.Epifanio, R.Navarro and E. Simoncelli,
%             Non-linear Image Representation for Efficient Perceptual Coding.
%             IEEE Trans. Im. Proc. Vol. 15, 1, pp 68-80 (2006)
%
%   [Camps08] G. Camps, J. Gutiérrez, G. Gómez and J. Malo
%             On the Suitable Domain for SVM Training in Image Coding.
%             Journal of Machine Learning Research, Vol. 9, pp 49-66 (2008)
%

% Load image

load images/barbara256   % The name of the image variable is Im (256*256 double array in the range [0,255])

% Encode the image (KeCoDe_encoder_achrom.m)
%
% The loop below runs over the available algorithms (see help of
% KeCoDe_encoder_achrom.m) to generate the encoded version of the image (at
% the specified target entropy) in different files.

Target_entropy=0.55; % Desired entropy (in bits/pix)

file_name='demo1';   % Note that algorithm identification will be added to this file name

% Output folder
you_are_here=pwd;
output_folder=which('KeCoDe_coder_tools_demo_achrom');
output_folder=output_folder(1:end-32);
cd(output_folder);
cd('tmp');
output_folder=pwd;
barra=output_folder(end-3);
output_folder=[output_folder,barra];
cd(you_are_here);
warning off

Num_iterat=10;       % Number of iterations used to achieve the desired file size
% (if this number is reduced to 1, a too crude
% approximation to the target entropy is obtained

for i=1:6
    Results = KeCoDe_encoder_achrom(Im,i,output_folder,file_name,Target_entropy,Num_iterat);
    eval(['Results',num2str(i),'=Results;'])
    clear Results
end

% Decode the files (KeCoDe_decoder_achrom.m)
%
% The loop below runs over the files generated by the previous loop
% available algorithms to decode them and obtain the decompressed
% images.

names=struct('name',[]);                 % Algorithm identifiers given by KeCoDe_encoder_achrom.m
names(1).name='_jpeg91_entropy_1';
names(2).name='_malo99_entropy_1';
names(3).name='_malo06_entropy_1';
names(4).name='_robinson03_entropy_1';
names(5).name='_gomez05_entropy_1';
names(6).name='_camps08_entropy_1';

for i=1:6
    file=[file_name,names(i).name];
    im=KeCoDe_decoder_achrom(output_folder,file);
    eval(['Im',num2str(i),'=im;'])
    clear im
end

figure,colormap gray,imshow(Im/255);title('Original.  Entropy=8 bpp')
figure,colormap gray
subplot(231),imshow(Results1.Image/255),title(['JPEG-like quantiz.  Entropy=',num2str(Results1.Entropy),' bpp'])
xlabel(char(['  RMSE=',num2str(round(10*Results1.RMSE)/10),'   SSIM=',num2str(round(100*Results1.SSIM)/100)],['  d_{DCT}=',num2str(round(10*Results1.MPE_non_linear)/10)]))

subplot(232),imshow(Results2.Image/255),title(['Malo 99.  Entropy=',num2str(Results2.Entropy),' bpp'])
xlabel(char(['  RMSE=',num2str(round(10*Results2.RMSE)/10),'   SSIM=',num2str(round(100*Results2.SSIM)/100)],['  d_{DCT}=',num2str(round(10*Results2.MPE_non_linear)/10)]))

subplot(233),imshow(Results3.Image/255),title(['Malo 06.  Entropy=',num2str(Results3.Entropy),' bpp'])
xlabel(char(['  RMSE=',num2str(round(10*Results3.RMSE)/10),'   SSIM=',num2str(round(100*Results3.SSIM)/100)],['  d_{DCT}=',num2str(round(10*Results3.MPE_non_linear)/10)]))

subplot(234),imshow(Results4.Image/255),title(['Robinson 05.  Entropy=',num2str(Results4.Entropy),' bpp'])
xlabel(char(['  RMSE=',num2str(round(10*Results4.RMSE)/10),'   SSIM=',num2str(round(100*Results4.SSIM)/100)],['  d_{DCT}=',num2str(round(10*Results4.MPE_non_linear)/10)]))

subplot(235),imshow(Results5.Image/255),title(['Gomez 05.  Entropy=',num2str(Results5.Entropy),' bpp'])
xlabel(char(['  RMSE=',num2str(round(10*Results5.RMSE)/10),'   SSIM=',num2str(round(100*Results5.SSIM)/100)],['  d_{DCT}=',num2str(round(10*Results5.MPE_non_linear)/10)]))

subplot(236),imshow(Results6.Image/255),title(['Camps 08.  Entropy=',num2str(Results6.Entropy),' bpp'])
xlabel(char(['  RMSE=',num2str(round(10*Results6.RMSE)/10),'   SSIM=',num2str(round(100*Results6.SSIM)/100)],['  d_{DCT}=',num2str(round(10*Results6.MPE_non_linear)/10)]))
