function KeCoDe_image_degradation_demo

%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                       %
%%  KeCoDe  Demo 3:      %
%%  Image Degradation    %
%%                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This script illustrates the use of 'KeCoDe_degradation.m'.
% It implements a set of controlled degradations:
%
%    * Gaussian blur (of tunable cut-off frequency) plus Gaussian colored
%      noise (of tunable frequency band and variance).
%      With appropriate parameters you can generate degradations ranging
%      from pure blurring to pure denoising (including white or colored noise).
%
%    * JPEG degradation: an example of non-Gaussian noise
%
%    * Salt & Pepper: another non-Gaussian noise
%
% The parameters will be set to obtain similar MSE but quite different
% visual appearance.

warning off

%%%%%%%%%%%%%%%%%%
%                %
% Original Image %
%                %
%%%%%%%%%%%%%%%%%%

load barbara256   % The image variable is named Im

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                        %
% BLUR + WHITE OR COLORED GAUSSIAN NOISE %
%                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% EXAMPLES
% --------
%
% (1) No blur + Gaussian white noise
% (2) No blur + Gaussian colored noise
% (3) Moderate blur + Gaussian colored noise
% (4) Severe blur + Gaussian colored noise

% (1) No blur + Gaussian white noise

deg_model='wt_gn';
variance=205;

[Imd,PSF] = KeCoDe_degradation(Im,deg_model,variance);

figure,colormap gray,imshow(Imd/255);axis square,
title(['Gaussian White Noise  \sigma^2=',num2str(variance)])
rmse=sqrt(sum(sum((Im-Imd).^2)))/256;
[rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Imd);
xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})

% For additional distortion measures, installDivisive Normalization Metric:
%  http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
%
% d=div_norm_metric(Im,Id);
% xlabel(['MSE^{1/2}=',num2str(round(rmse*10)/10),'  d_{wav}=',num2str(d)])

% (2) No blur + Gaussian colored noise

deg_model='col_gn';
band_limits_noise=[0.25 0.5];
variance=205;

[Imd,PSF] = KeCoDe_degradation(Im,deg_model,band_limits_noise,variance);

figure,colormap gray,imshow(Imd/255);axis square,
title(['Gaussian Colored Noise  \sigma^2=',num2str(variance)])
rmse=sqrt(sum(sum((Im-Imd).^2)))/256;
[rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Imd);
xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})

% For additional distortion measures, installDivisive Normalization Metric:
%  http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
%
% d=div_norm_metric(Im,Id);
% xlabel(['MSE^{1/2}=',num2str(round(rmse*10)/10),'  d_{wav}=',num2str(d)])

% (3) Moderate blur + Gaussian colored noise

deg_model='blur_col_gn';
cut_off_blur=0.6;
min_val_blur=0.001;
band_limits_noise=[0.25 0.5];
variance=160;

[Imd,PSF] = KeCoDe_degradation(Im,deg_model,cut_off_blur,min_val_blur,band_limits_noise,variance);

figure,colormap gray,imshow(Imd/255);axis square,
title(['Moderate blur  f_{cut}=',num2str(cut_off_blur), ' + Gaussian Colored Noise  \sigma^2=',num2str(variance)])
rmse=sqrt(sum(sum((Im-Imd).^2)))/256;
[rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Imd);
xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})

% For additional distortion measures, installDivisive Normalization Metric:
%  http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
%
% d=div_norm_metric(Im,Id);
% xlabel(['MSE^{1/2}=',num2str(round(rmse*10)/10),'  d_{wav}=',num2str(d)])

% (4) Severe blur + Gaussian colored noise

deg_model='blur_col_gn';
cut_off_blur=0.4;
min_val_blur=0.001;
band_limits_noise=[0.25 0.5];
variance=17;

[Imd,PSF] = KeCoDe_degradation(Im,deg_model,cut_off_blur,min_val_blur,band_limits_noise,variance);

figure,colormap gray,imshow(Imd/255);axis square,
title(['Severe blur  f_{cut}=',num2str(cut_off_blur), ' + Gaussian Colored Noise  \sigma^2=',num2str(variance)])
rmse=sqrt(sum(sum((Im-Imd).^2)))/256;
[rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Imd);
xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})

% For additional distortion measures, installDivisive Normalization Metric:
%  http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
%
% d=div_norm_metric(Im,Id);
% xlabel(['MSE^{1/2}=',num2str(round(rmse*10)/10),'  d_{wav}=',num2str(d)])

%%%%%%%%%%%%%%%%%%%%
%                  %
% JPEG DEGRADATION %
%                  %
%%%%%%%%%%%%%%%%%%%%

% In JPEG the quality (and hence the compresion ratio) is controlled by the
% parameter 'Quality' in the range [0, 100].
%
% In order to obtain challenging degradations for denoising algorithms, set
% this value in the range [1, 15]

deg_model='jpeg';
Quality = 7;

[Imd,PSF] = KeCoDe_degradation(Im,deg_model,Quality);

figure,colormap gray,
imshow(Imd/255);axis square,title(['JPEG Image with quality factor ' num2str(Quality)])
rmse=sqrt(sum(sum((Im-Imd).^2)))/256;
[rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Imd);
xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})

% For additional distortion measures, installDivisive Normalization Metric:
%  http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
%
% d=div_norm_metric(Im,Id);
% xlabel(['MSE^{1/2}=',num2str(round(rmse*10)/10),'  d_{wav}=',num2str(d)])

%%%%%%%%%%%%%%%%%
%               %
% SALT & PEPPER %
%               %
%%%%%%%%%%%%%%%%%

% Salt & Pepper noise consists on changing arbitrary pixels by the minimum
% and maximum possible values of the signal (e.g., 0 and 255 in 8-bit images)
%
% This function uses 'imnoise.m': the amount of corrupted pixels is given
% by the parameter D, which ranges in the interval [0,1]. The number of
% affected pixels is approximately D*num_pixels.
%

deg_model='salt_pepper';
D=0.011;
[Imd,PSF] = KeCoDe_degradation(Im,deg_model,D);

figure,colormap gray,imshow(Imd/255);axis square,title(['Salt & Pepper noise. Percentage affected=' num2str(D*100) '%'])
rmse=sqrt(sum(sum((Im-Imd).^2)))/256;
[rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Imd);
xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})

% For additional distortion measures, installDivisive Normalization Metric:
%  http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
%
% d=div_norm_metric(Im,Id);
% xlabel(['MSE^{1/2}=',num2str(round(rmse*10)/10),'  d_{wav}=',num2str(d)])
