function KeCoDe_image_denoising_demo

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                       %
%%  KeCoDe  Demo 4:                      %
%%  Wavelet and Kernel Image Denoising   %
%%                                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% In this demo, different wavelet-based denoising algorithms
% based on different assumptions on natural images statistics
% are compared in denoising problems using (1) high variance white
% Gaussian noise, and (2) JPEG noise (which is non-Gaussian, colored
% and signal dependent). Classical algorithms require the knowledge
% of variance noise. In the case of unknown variance, it is estimated by
% 'function_stdEst2D.m' (see Foi06 for details).
%
% The original image is corrupted by using 'KeCoDe_degradation.m'
% Denoising is performed using different options in 'KeCoDe_denoising.m'
% see it for further details:
%
%   * Hard Thresholding [Donoho95,Simoncelli99].... model 1
%
%   * Soft Thresholding [Donoho95,Simoncelli99].... model 2
%
%   * Bayesian denoising assuming [Figueiredo01].... model 3
%
%   * Kernel regularization in the wavelet domain with kernels inspired in
%     mutual information measures [Laparra10] ..... model 4
%
%     NOTE: The function 'mi_kernel_denoising.m' admits precomputed parameters
%     in order to apply the algorithm in different kinds of noise. See the
%     JPEG example below.
%
%
% [Laparra10]    V. Laparra, J. Gutierrez, G. Camps-Valls and J. Malo
%                Image Denoising with Kernels based on Natural Image Relations
%                Journal of Machine Learning Research (2010)
%
% [Donoho95]     David L. Donoho and Iain M. Johnstone. Adapting to unknown
%                smoothness via wavelet shrinkage. J. Am. Stat. Assoc., 90:1200-1224, 1995.
%
% [Simoncelli99] E. Simoncelli. Bayesian denoising of visual images in the wavelet
%                domain. In Bayesian Inference in Wavelet Based Models, pages 291-308.
%                Springer-Verlag, NY, 1999.
%
% [Figueiredo01] M. Figueiredo and R. Nowak. Wavelet-based image estimation:
%                an empirical Bayes approach using Jeffreys noninformative prior.
%                IEEE Transactions on Image Processing, 10(9):1322-1331, 2001.
%
% [Simoncelli97] E. Simoncelli. MatlabPyrTools. Matlab toolbox for wavelet transforms
%                http://www.cns.nyu.edu/~lcv/software.php
%
% [Foi06]        Alessandro Foi. Routine  'function_stdEst2D.m'  for automatic
%                noise variance estimation. Tampere University of
%                Technology - 2005-2006
%                http://www.cs.tut.fi/~lasip/2D

warning off

% Original Image
% ---------------
load barbara256

% Image degradation (see KeCoDe_image_degradation_demo.m)
% ------------------------------------------------
%
%     In the following loop over degradations select one of the following degradations (or both):
%
%     (1) White Gaussian noise
%     (2) JPEG noise
%

% Image distortion measures
% -------------------------
%
% If you have installed the following subjectively meaningful distortion measures:
%   * Divisive Normalization Metric
%      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
%   * Structural Similarity Index
%      http://www.cns.nyu.edu/~lcv/software.php
%   * S-CIELAB
%      http://white.stanford.edu/~brian/scielab/
% set the next 'meaningful_measures' variable to 1 to evaluate the distortion and
% the restoration results. By default meaningful_measures=0 (and just PSNR
% is given).

meaningful_measures=0;

for ii=1:2 % Loop over degradations

    degradation=ii

    if degradation==1

        %
        % Almost denoising problem
        %
        deg_model='blur_wt_gn';
        cut_off_blur=27.5/32;  % 32cpd is assumed to be the Nyquist frequency. 50 cpd is the cut-off frequency: no blur at all
        min_val_blur=0.05;
        variance=200;        % severe noise
        [Id1,PSF1] = KeCoDe_degradation(Im,deg_model,cut_off_blur,min_val_blur,variance);

        Id=Id1;
        PSF=PSF1;

        PSNR=psnr(Im,Id);
        if meaningful_measures==1
            % For subjectively meaningful distortion measures, please install:
            %   * Divisive Normalization Metric
            %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
            %   * Structural Similarity Index
            %      http://www.cns.nyu.edu/~lcv/software.php
            %   * S-CIELAB
            %      http://white.stanford.edu/~brian/scielab/
            %
            ssim=ssim_index(Im,Id);
            d=div_norm_metric(Im,Id);
            imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
            imr(:,:,1)=Id;imr(:,:,2)=Id;imr(:,:,3)=Id;
            SCIELab = calculo_scielab(imm,imr,1);

            figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title(['White Gaussian Noise (\sigma^2=400)']),
            %xlabel(['PSNR=',num2str(PSNR)])
            xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   SSIM=',num2str(round(100*ssim)/100)],['SCIELab=',num2str(round(100*SCIELab)/100),'  d_{wav}=',num2str(round(100*d)/100)]))

        else
            figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,
            %xlabel(['PSNR=',num2str(PSNR)])
            [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Id);
            title(['White Gaussian Noise (\sigma^2=',num2str(round((rmse^2)*10)/10),')']),
            xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})

        end
    else
        deg_model='jpeg';
        Q=7;
        [Id5,PSF5] = KeCoDe_degradation(Im,deg_model,Q);
        Id=Id5;
        PSF=PSF5;

        PSNR=psnr(Im,Id);
        if meaningful_measures==1
            % For subjectively meaningful distortion measures, please install:
            %   * Divisive Normalization Metric
            %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
            %   * Structural Similarity Index
            %      http://www.cns.nyu.edu/~lcv/software.php
            %   * S-CIELAB
            %      http://white.stanford.edu/~brian/scielab/
            %
            ssim=ssim_index(Im,Id);
            d=div_norm_metric(Im,Id);
            imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
            imr(:,:,1)=Id;imr(:,:,2)=Id;imr(:,:,3)=Id;
            SCIELab = calculo_scielab(imm,imr,1);

            figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title('JPEG Noise (Q=7)'),
            %xlabel(['PSNR=',num2str(PSNR)])
            xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   SSIM=',num2str(round(100*ssim)/100)],['SCIELab=',num2str(round(100*SCIELab)/100),'  d_{wav}=',num2str(round(100*d)/100)]))

        else
            figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title('JPEG Noise (Q=7)'),
            %xlabel(['PSNR=',num2str(PSNR)])
            [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Id);
            xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})

        end
    end

    % Image Denoising (see hardthres.m, softthres.m, bayesian_gauss_margin.m, mi_kernel_denoising.m)
    % ----------------------------------------------------------------------------------------------
    %

    tic

    %% HARD THRESHOLDING

    % Known variance or variance estimation
    if degradation==1
        va=variance;
    else        % Variance estimation for the JPEG case
        dev = function_stdEst2D(Id);
        va=dev^2;
    end
    disp('Hard Thresholding Image Denoising ...')
    imr_hard = KeCoDe_denoising(Id,1,va);
    PSNR=psnr(Im,imr_hard);

    if meaningful_measures==1
        % For subjectively meaningful distortion measures, please install:
        %   * Divisive Normalization Metric
        %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
        %   * Structural Similarity Index
        %      http://www.cns.nyu.edu/~lcv/software.php
        %   * S-CIELAB
        %      http://white.stanford.edu/~brian/scielab/
        %
        ssim=ssim_index(Im,imr_hard);d=div_norm_metric(Im,imr_hard);
        imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
        imr(:,:,1)=imr_hard;imr(:,:,2)=imr_hard;imr(:,:,3)=imr_hard;
        SCIELab = calculo_scielab(imm,imr,1);
        dis_hard=[PSNR ssim d SCIELab];
    else
        [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,imr_hard);
        dis_hard=[rmse SSIM d_DCT];
        
    end

    %% SOFT THRESHOLDING

    % Known variance or variance estimation
    if degradation==1
        va=variance;
    else        % Variance estimation for the JPEG case
        dev = function_stdEst2D(Id);
        va=dev^2;
    end
    disp('Soft Thresholding Image Denoising ...')
    imr_soft = KeCoDe_denoising(Id,2,va);
    PSNR=psnr(Im,imr_soft);
    if meaningful_measures==1
        % For subjectively meaningful distortion measures, please install:
        %   * Divisive Normalization Metric
        %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
        %   * Structural Similarity Index
        %      http://www.cns.nyu.edu/~lcv/software.php
        %   * S-CIELAB
        %      http://white.stanford.edu/~brian/scielab/
        %
        ssim=ssim_index(Im,imr_soft);d=div_norm_metric(Im,imr_soft);
        imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
        imr(:,:,1)=imr_soft;imr(:,:,2)=imr_soft;imr(:,:,3)=imr_soft;
        SCIELab = calculo_scielab(imm,imr,1);
        dis_soft=[PSNR ssim d SCIELab];
    else
        %dis_soft=[PSNR];
        [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,imr_soft);
        dis_soft=[rmse SSIM d_DCT];        
    end

    %% BAYESIAN DENOISING

    % Known variance or variance estimation
    if degradation==1
        va=variance;
    else        % Variance estimation for the JPEG case
        dev = function_stdEst2D(Id);
        va=dev^2;
    end
    disp('Bayesian Image Denoising assuming Gaussian signal and noise...')
    imr_bayes = KeCoDe_denoising(Id,3,va);
    PSNR=psnr(Im,imr_bayes);
    if meaningful_measures==1
        % For subjectively meaningful distortion measures, please install:
        %   * Divisive Normalization Metric
        %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
        %   * Structural Similarity Index
        %      http://www.cns.nyu.edu/~lcv/software.php
        %   * S-CIELAB
        %      http://white.stanford.edu/~brian/scielab/
        %
        ssim=ssim_index(Im,imr_bayes);d=div_norm_metric(Im,imr_bayes);
        imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
        imr(:,:,1)=imr_bayes;imr(:,:,2)=imr_bayes;imr(:,:,3)=imr_bayes;
        SCIELab = calculo_scielab(imm,imr,1);
        dis_bayes=[PSNR ssim d SCIELab];
    else
        %dis_bayes=[PSNR];
        [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,imr_bayes);
        dis_bayes=[rmse SSIM d_DCT];        
    end

    %% KERNEL DENOISING

    % Parameter settings: epsilon_profile and (epsilon,C,S) values
    if degradation==1
        % KeCoDe_denoising model 4 performs denoising using the kernel
        % method assuming Gaussian noise
         disp('Mutual Info Kernel Image Denoising ...')
        imr_kernel = KeCoDe_denoising(Id,4,variance);
    elseif degradation==2
        % Particular parameters have to be used depending on the noise
        C       = 3000;
        S       = 1;
        epsilon = 1.7;
        load Epsilon_profile_JPEG_Q7.mat
        epsilon_profile  = perfil_eps;
        disp('Mutual Info Kernel Image Denoising ...')
        imr_kernel = mi_kernel_denoising(Id,epsilon,C,S,epsilon_profile);
    end

    PSNR=psnr(Im,imr_kernel);
    if meaningful_measures==1
        % For subjectively meaningful distortion measures, please install:
        %   * Divisive Normalization Metric
        %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
        %   * Structural Similarity Index
        %      http://www.cns.nyu.edu/~lcv/software.php
        %   * S-CIELAB
        %      http://white.stanford.edu/~brian/scielab/
        %
        ssim=ssim_index(Im,imr_kernel);d=div_norm_metric(Im,imr_kernel);
        imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
        imr(:,:,1)=imr_kernel;imr(:,:,2)=imr_kernel;imr(:,:,3)=imr_kernel;
        SCIELab = calculo_scielab(imm,imr,1);
        dis_kernel=[PSNR ssim d SCIELab];
    else
        % dis_kernel=[PSNR];
        [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,imr_kernel);
        dis_kernel=[rmse SSIM d_DCT];        
    end
    disp('Done!')

    dis(:,:,ii)=[dis_hard;dis_soft;dis_bayes;dis_kernel];

    if meaningful_measures==1
        % If you installed Divisive Normalization Metric, SSIM and S-CIELab
        %
        figure,subplot(221),imshow(imr_hard(33:end,33:end)/255),title('Hard'),xlabel(char(['  PSNR=',num2str(round(10*dis(1,1,ii))/10),'   SSIM=',num2str(round(100*dis(1,2,ii))/100)],['SCIELab=',num2str(round(100*dis(1,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(1,3,ii))/100)]))
        subplot(222),imshow(imr_soft(33:end,33:end)/255),title('Soft'),xlabel(char(['  PSNR=',num2str(round(10*dis(2,1,ii))/10),'   SSIM=',num2str(round(100*dis(2,2,ii))/100)],['SCIELab=',num2str(round(100*dis(2,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(2,3,ii))/100)]))
        subplot(223),imshow(imr_bayes(33:end,33:end)/255),title('Bayesian with Gaussian Marginals'),xlabel(char(['  PSNR=',num2str(round(10*dis(3,1,ii))/10),'   SSIM=',num2str(round(100*dis(3,2,ii))/100)],['SCIELab=',num2str(round(100*dis(3,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(3,3,ii))/100)]))
        subplot(224),imshow(imr_kernel(33:end,33:end)/255),title('Kernel'),xlabel(char(['  PSNR=',num2str(round(10*dis(4,1,ii))/10),'   SSIM=',num2str(round(100*dis(4,2,ii))/100)],['SCIELab=',num2str(round(100*dis(4,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(4,3,ii))/100)]))
    else
        figure,
        subplot(221),imshow(imr_hard(33:end,33:end)/255),title('Hard'),
        % xlabel(char(['  PSNR=',num2str(round(10*dis(1,1,ii))/10)]));
        xlabel({['MSE^{1/2} = ',num2str(round(dis(1,1,ii)*10)/10),'  SSIM = ',num2str(round(dis(1,2,ii)*100)/100)],['d_{DCT} = ',num2str(round(dis(1,3,ii)*10)/10)]})
        subplot(222),imshow(imr_soft(33:end,33:end)/255),title('Soft'),
        %xlabel(char(['  PSNR=',num2str(round(10*dis(2,1,ii))/10)]));
        xlabel({['MSE^{1/2} = ',num2str(round(dis(2,1,ii)*10)/10),'  SSIM = ',num2str(round(dis(2,2,ii)*100)/100)],['d_{DCT} = ',num2str(round(dis(2,3,ii)*10)/10)]})
        subplot(223),imshow(imr_bayes(33:end,33:end)/255),title('Bayesian with Gaussian Marginals'),
        %xlabel(char(['  PSNR=',num2str(round(10*dis(3,1,ii))/10)]));
        xlabel({['MSE^{1/2} = ',num2str(round(dis(3,1,ii)*10)/10),'  SSIM = ',num2str(round(dis(3,2,ii)*100)/100)],['d_{DCT} = ',num2str(round(dis(3,3,ii)*10)/10)]})
        subplot(224),imshow(imr_kernel(33:end,33:end)/255),title('Kernel'),
        %xlabel(char(['  PSNR=',num2str(round(10*dis(4,1,ii))/10)]));
        xlabel({['MSE^{1/2} = ',num2str(round(dis(4,1,ii)*10)/10),'  SSIM = ',num2str(round(dis(4,2,ii)*100)/100)],['d_{DCT} = ',num2str(round(dis(4,3,ii)*10)/10)]})
    end

    toc

end
