function KeCoDe_image_restoration_demo

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                  %
%%  KeCoDe  Demo 5:                                 %
%%  Image Restoration through local regularization  %
%%                                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% In this demo the different local regularization techniques
% implemented in KeCoDe_restoration.m are compared for different denoising-deblurring
% problems, Salt and Pepper noise and JPEG noise.
%
% Degradation is done by using 'KeCoDe_degradation.m'
% Restoration is done by using different options in 'KeCoDe_restoration.m':
%
%   'L2'  = Second derivative regularization operator [Tikhonov79]
%   'CSF' = Regularization operator using a linear perception model [Hunt75]
%   'AR4' = Regularization by spectrum estimation using 4th order
%           Auto-Regressive model [Banham&Katssaggelos97]
%   'AR8' = Regularization by spectrum estimation using 8th order
%           Auto-Regressive model [Banham&Katssaggelos97]
%   'AR12'= Regularization by spectrum estimation using 12th order
%           Auto-Regressive model [Banham&Katssaggelos97]
%   'PER' = Regularization operator based on divisive normalization
%           (non-linear perception model) [Gutierrez et al. 06]
%
% See details at:    J. Gutierrez et al. Regularization Operators for
%                    Natural Images based on Non-linear Perception Models.
%                    IEEE Trans. Im. Proc. 15(1). 2006

warning off

% Original Image
% ---------------
load barbara256

% Image degradation (see KeCoDe_image_degradation_demo.m)
% ------------------------------------------------
%
%     In the following loop select one (or several) of the following degradations:
%
%     (1) Slight blur + White Gaussian noise (almost denoising problem)
%     (2) Moderate blur + White Gaussian noise (Deconvolution of a noisy image)
%     (3) Salt & Pepper
%     (4) JPEG
%
% Image distortion measures
% -------------------------
%
% If you have installed the following subjectively meaningful distortion measures:
%   * Divisive Normalization Metric
%      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
%   * Structural Similarity Index
%      http://www.cns.nyu.edu/~lcv/software.php
%   * S-CIELAB
%      http://white.stanford.edu/~brian/scielab/
% set the next 'meaningful_measures' variable to 1 to evaluate the distortion and
% the restoration results. By default meaningful_measures=0 (and just PSNR
% is given).

meaningful_measures=0;

for ii=1:4   % Loop over distortions

    degradation=ii

    if degradation==1

        %
        % Almost denoising problem
        %
        deg_model='blur_wt_gn';
        cut_off_blur=27.2/32;  % 32cpd is assumed to be the Nyquist frequency. 27 cpd is the cut-off frequency: almost no blur
        min_val_blur=0.05;
        variance=200;        % severe noise
        [Id1,PSF1] = KeCoDe_degradation(Im,deg_model,cut_off_blur,min_val_blur,variance);

        Id=Id1;
        PSF=PSF1;

        PSNR=psnr(Im,Id);
        if meaningful_measures==1
            % For subjectively meaningful distortion measures, please install:
            %   * Divisive Normalization Metric
            %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
            %   * Structural Similarity Index
            %      http://www.cns.nyu.edu/~lcv/software.php
            %   * S-CIELAB
            %      http://white.stanford.edu/~brian/scielab/
            %
            ssim=ssim_index(Im,Id);
            d=div_norm_metric(Im,Id);
            imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
            imr(:,:,1)=Id;imr(:,:,2)=Id;imr(:,:,3)=Id;
            SCIELab = calculo_scielab(imm,imr,1);
            figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title('Blur (f_c=27 cpd) + White Gaussian Noise (\sigma^2=200)'),
            xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   SSIM=',num2str(round(100*ssim)/100)],['SCIELab=',num2str(round(100*SCIELab)/100),'  d_{wav}=',num2str(round(100*d)/100)]))

        else
            figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title('Blur (f_c=27 cpd) + White Gaussian Noise (\sigma^2=200)'),
            %xlabel(['PSNR=',num2str(PSNR)])
            %xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   SSIM=',num2str(round(100*ssim)/100)],['SCIELab=',num2str(round(100*SCIELab)/100),'  d_{wav}=',num2str(round(100*d)/100)]))
            [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Id);
            xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})

        end

    elseif degradation==2

        %
        % Deconvolution of a noisy image
        %
        deg_model='blur_wt_gn';
        cut_off_blur=16/32;
        min_val_blur=0.05;
        variance=100;
        [Id3,PSF3] = KeCoDe_degradation(Im,deg_model,cut_off_blur,min_val_blur,variance);

        Id=Id3;
        PSF=PSF3;

        PSNR=psnr(Im,Id);
        if meaningful_measures==1
            % For subjectively meaningful distortion measures, please install:
            %   * Divisive Normalization Metric
            %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
            %   * Structural Similarity Index
            %      http://www.cns.nyu.edu/~lcv/software.php
            %   * S-CIELAB
            %      http://white.stanford.edu/~brian/scielab/
            %
            ssim=ssim_index(Im,Id);
            d=div_norm_metric(Im,Id);
            imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
            imr(:,:,1)=Id;imr(:,:,2)=Id;imr(:,:,3)=Id;
            SCIELab = calculo_scielab(imm,imr,1);
            figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title('Blur (f_c=16 cpd) + White Gaussian Noise (\sigma^2=100)'),
            xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   SSIM=',num2str(round(100*ssim)/100)],['SCIELab=',num2str(round(100*SCIELab)/100),'  d_{wav}=',num2str(round(100*d)/100)]))

        else
            figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title('Blur (f_c=16 cpd) + White Gaussian Noise (\sigma^2=100)'),
            %xlabel(['PSNR=',num2str(PSNR)])
            %xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   SSIM=',num2str(round(100*ssim)/100)],['SCIELab=',num2str(round(100*SCIELab)/100),'  d_{wav}=',num2str(round(100*d)/100)]))
            [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Id);
            xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})

        end

    elseif degradation==3
        deg_model='salt_pepper';
        D=0.011;
        [Id4,PSF4] = KeCoDe_degradation(Im,deg_model,D);
        Id=Id4;
        PSF=PSF4;

        PSNR=psnr(Im,Id);
        if meaningful_measures==1
            % For subjectively meaningful distortion measures, please install:
            %   * Divisive Normalization Metric
            %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
            %   * Structural Similarity Index
            %      http://www.cns.nyu.edu/~lcv/software.php
            %   * S-CIELAB
            %      http://white.stanford.edu/~brian/scielab/
            %
            ssim=ssim_index(Im,Id);
            d=div_norm_metric(Im,Id);
            imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
            imr(:,:,1)=Id;imr(:,:,2)=Id;imr(:,:,3)=Id;
            SCIELab = calculo_scielab(imm,imr,1);
            figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title('Salt & Pepper Noise'),
            %xlabel(['PSNR=',num2str(PSNR)])
            xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   SSIM=',num2str(round(100*ssim)/100)],['SCIELab=',num2str(round(100*SCIELab)/100),'  d_{wav}=',num2str(round(100*d)/100)]))

        else
            figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title('Salt & Pepper Noise'),
            %xlabel(['PSNR=',num2str(PSNR)])
            %xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   SSIM=',num2str(round(100*ssim)/100)],['SCIELab=',num2str(round(100*SCIELab)/100),'  d_{wav}=',num2str(round(100*d)/100)]))
            [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Id);
            xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})

        end

    else
        deg_model='jpeg';
        Q=7;
        [Id5,PSF5] = KeCoDe_degradation(Im,deg_model,Q);
        Id=Id5;
        PSF=PSF5;

        PSNR=psnr(Im,Id);

        if meaningful_measures==1
            % For subjectively meaningful distortion measures, please install:
            %   * Divisive Normalization Metric
            %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
            %   * Structural Similarity Index
            %      http://www.cns.nyu.edu/~lcv/software.php
            %   * S-CIELAB
            %      http://white.stanford.edu/~brian/scielab/
            %
            ssim=ssim_index(Im,Id);
            d=div_norm_metric(Im,Id);
            imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
            imr(:,:,1)=Id;imr(:,:,2)=Id;imr(:,:,3)=Id;
            SCIELab = calculo_scielab(imm,imr,1);

            figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title('JPEG Noise (Q=7)'),
            xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   SSIM=',num2str(round(100*ssim)/100)],['SCIELab=',num2str(round(100*SCIELab)/100),'  d_{wav}=',num2str(round(100*d)/100)]))

        else

            figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title('JPEG Noise (Q=7)'),
            %xlabel(['PSNR=',num2str(PSNR)])
            %xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   SSIM=',num2str(round(100*ssim)/100)],['SCIELab=',num2str(round(100*SCIELab)/100),'  d_{wav}=',num2str(round(100*d)/100)]))
            [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Id);
            xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})

        end
    end

    % Image Restoration (see KeCoDe_restoration.m)
    % ---------------------------------
    %

    tic

    % Values of the regularization parameter (lambda) to set the relative weight
    % of regularization versus deviation from the observed data.
    % In every case (for every regularization functional) the appropriate
    % regularization parameter for each region will be chosen through
    % the L-curve method.
    %
    % The algorithms can be speed us by fixing a single regularization
    % parameter value (e.g., Nlambdas=1, lambdas=3)
    % In this simplified case the selected lambda will be applied to every
    % region of the image.

    Nlambdas = 60;
    lambdas = logspace(-4,3,Nlambdas);

    %% RESTORE IMAGE: SECOND DERIVATIVE OPERATOR

    disp(['Restoring image using L2 (' num2str(Nlambdas) ' values of lambda)']);

    imrl2=KeCoDe_restoration(Id,'L2',PSF,lambdas);

    PSNR=psnr(Im,imrl2);
    if meaningful_measures==1
        % For subjectively meaningful distortion measures, please install:
        %   * Divisive Normalization Metric
        %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
        %   * Structural Similarity Index
        %      http://www.cns.nyu.edu/~lcv/software.php
        %   * S-CIELAB
        %      http://white.stanford.edu/~brian/scielab/
        %
        ssim=ssim_index(Im,imrl2);d=div_norm_metric(Im,imrl2);
        imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
        imr(:,:,1)=imrl2;imr(:,:,2)=imrl2;imr(:,:,3)=imrl2;
        SCIELab = calculo_scielab(imm,imr,1);
        dis_l2=[PSNR ssim d SCIELab];
    else
        %dis_l2=[PSNR];
        [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,imrl2);
        dis_l2=[rmse SSIM d_DCT]; 
    end

    %% RESTORE IMAGE: CSF OPERATOR

    disp(['Restoring image using CSF (' num2str(Nlambdas) ' values of lambda)']);

    imrcsf=KeCoDe_restoration(Id,'CSF',PSF,lambdas);

    PSNR=psnr(Im,imrcsf);
    if meaningful_measures==1
        % For subjectively meaningful distortion measures, please install:
        %   * Divisive Normalization Metric
        %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
        %   * Structural Similarity Index
        %      http://www.cns.nyu.edu/~lcv/software.php
        %   * S-CIELAB
        %      http://white.stanford.edu/~brian/scielab/
        %
        ssim=ssim_index(Im,imrcsf);d=div_norm_metric(Im,imrcsf);
        imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
        imr(:,:,1)=imrcsf;imr(:,:,2)=imrcsf;imr(:,:,3)=imrcsf;
        SCIELab = calculo_scielab(imm,imr,1);
        dis_csf=[PSNR ssim d SCIELab];
    else
        %dis_csf=[PSNR];
        [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,imrcsf);
        dis_csf=[rmse SSIM d_DCT];         
    end

    %% RESTORE IMAGE: AR4 OPERATOR

    disp(['Restoring image using AR4 (' num2str(Nlambdas) ' values of lambda)']);

    imrar4=KeCoDe_restoration(Id,'AR4',PSF,lambdas);

    PSNR=psnr(Im,imrar4);
    if meaningful_measures==1
         % For subjectively meaningful distortion measures, please install:
            %   * Divisive Normalization Metric
            %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
            %   * Structural Similarity Index
            %      http://www.cns.nyu.edu/~lcv/software.php
            %   * S-CIELAB
            %      http://white.stanford.edu/~brian/scielab/
            %
        ssim=ssim_index(Im,imrar4);d=div_norm_metric(Im,imrar4);
        imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
        imr(:,:,1)=imrar4;imr(:,:,2)=imrar4;imr(:,:,3)=imrar4;
        SCIELab = calculo_scielab(imm,imr,1);
        dis_ar4=[PSNR ssim d SCIELab]
    else
        % dis_ar4=[PSNR];
        [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,imrar4);
        dis_ar4=[rmse SSIM d_DCT];
    end

    %% RESTORE IMAGE: AR8 OPERATOR

    disp(['Restoring image using AR8 (' num2str(Nlambdas) ' values of lambda)']);

    imrar8=KeCoDe_restoration(Id,'AR8',PSF,lambdas);

    PSNR=psnr(Im,imrar8);
    if meaningful_measures==1
          % For subjectively meaningful distortion measures, please install:
            %   * Divisive Normalization Metric
            %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
            %   * Structural Similarity Index
            %      http://www.cns.nyu.edu/~lcv/software.php
            %   * S-CIELAB
            %      http://white.stanford.edu/~brian/scielab/
            %
        ssim=ssim_index(Im,imrar8);d=div_norm_metric(Im,imrar8);
        imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
        imr(:,:,1)=imrar8;imr(:,:,2)=imrar8;imr(:,:,3)=imrar8;
        SCIELab = calculo_scielab(imm,imr,1);
        dis_ar8=[PSNR ssim d SCIELab];
    else
        % dis_ar8=[PSNR];
        [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,imrar8);
        dis_ar8=[rmse SSIM d_DCT];
    end

    %% RESTORE IMAGE: AR12 OPERATOR

    disp(['Restoring image using AR12 (' num2str(Nlambdas) ' values of lambda)']);

    imrar12=KeCoDe_restoration(Id,'AR12',PSF,lambdas);

    PSNR=psnr(Im,imrar12);
    if meaningful_measures==1
          % For subjectively meaningful distortion measures, please install:
            %   * Divisive Normalization Metric
            %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
            %   * Structural Similarity Index
            %      http://www.cns.nyu.edu/~lcv/software.php
            %   * S-CIELAB
            %      http://white.stanford.edu/~brian/scielab/
            %
        ssim=ssim_index(Im,imrar12);d=div_norm_metric(Im,imrar12);
        imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
        imr(:,:,1)=imrar12;imr(:,:,2)=imrar12;imr(:,:,3)=imrar12;
        SCIELab = calculo_scielab(imm,imr,1);
        dis_ar12=[PSNR ssim d SCIELab];
    else
        % dis_ar12=[PSNR];
        [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,imrar12);
        dis_ar12=[rmse SSIM d_DCT];        
    end

    %% RESTORE IMAGE: PERCEPTUAL OPERATOR

    disp(['Restoring image using PERCEPTUAL (' num2str(Nlambdas) ' values of lambda)']);

    imrper=KeCoDe_restoration(Id,'PER',PSF,lambdas);

    PSNR=psnr(Im,imrper);
    if meaningful_measures==1
         % For subjectively meaningful distortion measures, please install:
            %   * Divisive Normalization Metric
            %      http://www.uv.es/vista/vistavalencia/div_norm_metric/div_norm_metric.zip
            %   * Structural Similarity Index
            %      http://www.cns.nyu.edu/~lcv/software.php
            %   * S-CIELAB
            %      http://white.stanford.edu/~brian/scielab/
            %
        ssim=ssim_index(Im,imrper);d=div_norm_metric(Im,imrper);
        imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
        imr(:,:,1)=imrper;imr(:,:,2)=imrper;imr(:,:,3)=imrper;
        SCIELab = calculo_scielab(imm,imr,1);
        dis_per=[PSNR ssim d SCIELab];
    else
        % dis_per=[PSNR];
        [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,imrper);
        dis_per=[rmse SSIM d_DCT];        
    end

    dis(:,:,ii)=[dis_l2;dis_csf;dis_ar4;dis_ar8;dis_ar12;dis_per];

    if meaningful_measures==0
        % If PSNR is the only distortion
        %
        figure,subplot(231),imshow(imrl2/255),title('L2'),
        %xlabel(['PSNR=',num2str(round(10*dis(1,1,ii))/10)])
        xlabel({['MSE^{1/2} = ',num2str(round(dis(1,1,ii)*10)/10),'  SSIM = ',num2str(round(dis(1,2,ii)*100)/100)],['d_{DCT} = ',num2str(round(dis(1,3,ii)*10)/10)]})
        subplot(232),imshow(imrcsf/255),title('CSF'),
        %xlabel(['PSNR=',num2str(round(10*dis(2,1,ii))/10)])
        xlabel({['MSE^{1/2} = ',num2str(round(dis(2,1,ii)*10)/10),'  SSIM = ',num2str(round(dis(2,2,ii)*100)/100)],['d_{DCT} = ',num2str(round(dis(2,3,ii)*10)/10)]})
        subplot(233),imshow(imrar4/255),title('AR4'),
        %xlabel(['PSNR=',num2str(round(10*dis(3,1,ii))/10)])
        xlabel({['MSE^{1/2} = ',num2str(round(dis(3,1,ii)*10)/10),'  SSIM = ',num2str(round(dis(3,2,ii)*100)/100)],['d_{DCT} = ',num2str(round(dis(3,3,ii)*10)/10)]})
        subplot(234),imshow(imrar8/255),title('AR8'),
        %xlabel(['PSNR=',num2str(round(10*dis(4,1,ii))/10)])
        xlabel({['MSE^{1/2} = ',num2str(round(dis(4,1,ii)*10)/10),'  SSIM = ',num2str(round(dis(4,2,ii)*100)/100)],['d_{DCT} = ',num2str(round(dis(4,3,ii)*10)/10)]})
        subplot(235),imshow(imrar12/255),title('AR12'),
        %xlabel(['PSNR=',num2str(round(10*dis(5,1,ii))/10)])
        xlabel({['MSE^{1/2} = ',num2str(round(dis(5,1,ii)*10)/10),'  SSIM = ',num2str(round(dis(5,2,ii)*100)/100)],['d_{DCT} = ',num2str(round(dis(5,3,ii)*10)/10)]})
        subplot(236),imshow(imrper/255),title('Perceptual'),
        %xlabel(['PSNR=',num2str(round(10*dis(6,1,ii))/10)])
        xlabel({['MSE^{1/2} = ',num2str(round(dis(6,1,ii)*10)/10),'  SSIM = ',num2str(round(dis(6,2,ii)*100)/100)],['d_{DCT} = ',num2str(round(dis(6,3,ii)*10)/10)]})

    else
        % If you installed the Divisive Normalization Metric, SSIM and S-CIELab
        %
        figure,subplot(231),imshow(imrl2(33:end,33:end)/255),title('L2'),xlabel(char(['  PSNR=',num2str(round(10*dis(1,1,ii))/10),'   SSIM=',num2str(round(100*dis(1,2,ii))/100)],['SCIELab=',num2str(round(100*dis(1,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(1,3,ii))/100)]))
        subplot(232),imshow(imrcsf(33:end,33:end)/255),title('CSF'),xlabel(char(['  PSNR=',num2str(round(10*dis(2,1,ii))/10),'   SSIM=',num2str(round(100*dis(2,2,ii))/100)],['SCIELab=',num2str(round(100*dis(2,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(2,3,ii))/100)]))
        subplot(233),imshow(imrar4(33:end,33:end)/255),title('AR4'),xlabel(char(['  PSNR=',num2str(round(10*dis(3,1,ii))/10),'   SSIM=',num2str(round(100*dis(3,2,ii))/100)],['SCIELab=',num2str(round(100*dis(3,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(3,3,ii))/100)]))
        subplot(234),imshow(imrar8(33:end,33:end)/255),title('AR8'),xlabel(char(['  PSNR=',num2str(round(10*dis(4,1,ii))/10),'   SSIM=',num2str(round(100*dis(4,2,ii))/100)],['SCIELab=',num2str(round(100*dis(4,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(4,3,ii))/100)]))
        subplot(235),imshow(imrar12(33:end,33:end)/255),title('AR12'),xlabel(char(['  PSNR=',num2str(round(10*dis(5,1,ii))/10),'   SSIM=',num2str(round(100*dis(5,2,ii))/100)],['SCIELab=',num2str(round(100*dis(5,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(5,3,ii))/100)]))
        subplot(236),imshow(imrper(33:end,33:end)/255),title('Perceptual'),xlabel(char(['  PSNR=',num2str(round(10*dis(6,1,ii))/10),'   SSIM=',num2str(round(100*dis(6,2,ii))/100)],['SCIELab=',num2str(round(100*dis(6,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(6,3,ii))/100)]))
    end
    toc

end
